
/*
 * HPTalx
 * A PC<->HP48/49 communication program for Linux
 *
 * (C) 1999, 2000 by Bruno Barberi Gnecco
 * (C) 2005 by Bruno Barberi Gnecco, Rafael Ostertag
 *
 * $Id: gui.c,v 1.3 2007/09/29 21:21:01 guengel Exp $
 *
 *
 * Copyright note:
 *
 * This program is free software; you can
 * redistribute it and/or modify it under the
 * terms of the GNU General Public License as
 * published by the Free Software Foundation;
 * either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that
 * it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU
 * General Public License along with this
 * program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "hptalx.h"
#include "gui.h"
#include "gui_utils.h"
#include "messages.h"
#include "io.h"
#include "setup.h"

#include <gdk/gdkkeysyms.h>
#include <string.h>

/*
 * This flag is used by gui_command_window_create() to determine whether or not
 * the window already has been created. So we can avoid several creation of
 * the same window 
 */
static gboolean     command_window_created = FALSE;

/*
 * This is the struct passed to _gui_command_window_prepare_args() in order to
 * setup the arguments for the signal handler passed by the user to
 * gui_command_window_create() in send_command_handler 
 */
struct _command_window_internal_args
{
    GtkWidget          *input;
    command_window_send_args *args;
};
typedef struct _command_window_internal_args command_window_internal_args;

GtkWidget          *dir_pc;
GtkWidget          *dir_hp;
GtkWidget          *files_pc;
GtkWidget          *files_hp;
GtkWidget          *pop_up_menu;
GtkItemFactory     *item_factory;

static gint         _gui_pop_menu (GtkWidget * widget, GdkEvent * event);
static void         _gui_set_transfer_type (GtkWidget * w, gpointer data);

static void         _gui_cancel_transfer (GtkWidget * w, gpointer data);

static void         _gui_backup_ok (GtkWidget * w, gpointer data);
static void         _gui_backup_cancel (GtkWidget * w, gpointer data);

static void         _gui_quit_command_window (GtkWidget * w, gpointer data);
static void         _gui_command_window_prepare_args (GtkWidget * w,
						      gpointer data);
static void         _gui_clear_input_entry (GtkWidget * w, gpointer data);

static              gint
_gui_dir_select_row (GtkWidget * widget, GdkEventButton * event,
		     gpointer which);

static gchar       *_gui_dir_list_get_name_selected (GtkTreeView * list);

static gint         _gui_file_select_row (GtkWidget * widget,
					  GdkEventButton * event,
					  gpointer which);
static gchar       *_gui_file_list_get_type_str_selected (GtkTreeView * list);
static gchar       *_gui_file_list_get_name_selected (GtkTreeView * list);

/*--------------------------------------------------------------------------*/

/********************************************/

/********************************************/

/*******************      *******************/

/******************* Menu *******************/

/*******************      *******************/

/********************************************/

/********************************************/

GtkItemFactoryEntry items[] = {
    {"/_File", NULL, NULL, 0, "<Branch>"},
    {"/File/_Transfer", "<control>T", io_transfer, TRUE	/* Refresh after * *
							 * transfer */ , NULL},
    {"/File/_Move", "<control>M", io_move, 0, NULL},
    {"/File/sep1", NULL, NULL, 0, "<Separator>"},
    {"/File/_Edit", "<control>E", io_edit, 0, NULL},
    {"/File/_Copy...", "<control>C", io_copy, 0, NULL},
    {"/File/_Rename...", "<control>R", io_renamew, 0, NULL},
    {"/File/_Unpack", "<control>P", io_unpack, 0, NULL},
    {"/File/_Delete", "<control>L", io_delete, TRUE	/* Refresh after
							 * deletion */ , NULL},
    {"/File/sep2", NULL, NULL, 0, "<Separator>"},
    {"/File/Select _all", "<control>A", gui_select_all_file_list_item, 0,
     NULL},
    {"/File/_Unselect all", "<control>U", gui_unselect_all_file_list_item, 0,
     NULL},
    {"/File/sep3", NULL, NULL, 0, "<Separator>"},
    {"/File/Re_fresh", "<control>F", io_refreshall, 0, NULL},
    {"/File/_Setup...", "<control>N", setup, 0, NULL},
    {"/File/sep4", NULL, NULL, 0, "<Separator>"},
    {"/File/_Quit", "<control>Q", gtk_main_quit, 0, NULL},

    {"/_Directories", NULL, NULL, 0, "<Branch>"},
    {"/Directories/C_reate Directory...", NULL, io_create_dir, 0, NULL},
    {"/Directories/C_hange Directory", NULL, io_change_dir, 0, NULL},
    {"/Directories/_Remove Directory", NULL, io_remove_dir, TRUE, NULL},
    {"/_Utilities", NULL, NULL, 0, "<Branch>"},
    {"/Utilities/_Command...", NULL, io_command, 0, NULL},
#ifdef UPDATE_ROM
    {"/Utilities/_Update ROM", NULL, update_rom, 0, NULL},
#endif
    {"/Utilities/sep1", NULL, NULL, 0, "<Separator>"},
    {"/Utilities/_Backup...", NULL, io_backup, 0, NULL},
    {"/Utilities/_Restore", NULL, io_restore, 0, NULL},
    {"/Utilities/sep1", NULL, NULL, 0, "<Separator>"},

    {"/Utilities/_Transfer type", NULL, NULL, 0, "<Branch>"},
    {"/Utilities/Transfer type/Auto", NULL, NULL, 0, "<RadioItem>"},
    {"/Utilities/Transfer type/Binary", NULL, NULL, 0, "<RadioItem>"},
    {"/Utilities/Transfer type/ASCII", NULL, NULL, 0, "<RadioItem>"},

    {"/_Connect", NULL, NULL, 0, "<Branch>"},
    {"/Connect/_Connect", "<control>B", io_connect, 0, NULL},
    {"/Connect/_Disconnect", "<control>D", io_disconnect,
     K_FULL_QUIT, NULL},

    {"/_Help", NULL, NULL, 0, "<LastBranch>"},
    {"/Help/How to use", NULL, gui_howtouse, 0, NULL},
    {"/Help/About", NULL, gui_about, 0, NULL}
};

const char         *on_items[] = { "<main>/Utilities/Command...",
    "<main>/Connect/Disconnect", "<main>/File/Transfer",
    "<main>/File/Move", "<main>/Utilities/Backup...",
    "<main>/Utilities/Restore", NULL
};
const char         *off_items[] = { "<main>/Connect/Connect", NULL };
const char         *hp49_items[] = {
    /*
     * "<main>/Utilities/Protocol/XModem",
     */ NULL
};

/*
 * adapted from GTK's docs/examples 
 */
static              gint
_gui_pop_menu (GtkWidget * widget, GdkEvent * event)
{
    if (event->type == GDK_BUTTON_PRESS) {
	GdkEventButton     *bevent = (GdkEventButton *) event;

	if (bevent->button == 3)	/* left */
	    gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL,
			    NULL, bevent->button, bevent->time);
    }
    return FALSE;
}

/*
 * this function has been adapted from GTK's
 * tutorial. 
 */
void
gui_get_main_menu (GtkWidget * window, GtkWidget ** menubar)
{
    GtkAccelGroup      *accel_group;
    GtkWidget          *aut;
    GtkWidget          *bin;
    GtkWidget          *ascii;
    gint                nmenu_items;

    nmenu_items = sizeof (items) / sizeof (items[0]);
    accel_group = gtk_accel_group_new ();

    /*
     * This function initializes the item factory.
     * Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
     * or GTK_TYPE_OPTION_MENU.
     * Param 2: The path of the menu.
     * Param 3: A pointer to a gtk_accel_group.  The item factory sets up
     * the accelerator table while generating menus.
     */
    item_factory =
	gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);

    /*
     * This function generates the menu items. Pass the item factory,
     * the number of items in the array, the array itself, and any
     * callback data for the the menu items. 
     */
    gtk_item_factory_create_items (item_factory, nmenu_items, items, NULL);

    /*
     * Attach the new accelerator group to the window. 
     */
    gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

    /*
     * connect signals and lists to radio menu items 
     */

    /*
     * transfer type 
     */
    aut =
	gtk_item_factory_get_widget (item_factory,
				     "<main>/Utilities/Transfer type/Auto");
    g_signal_connect (GTK_OBJECT (aut), "toggled",
		      G_CALLBACK (_gui_set_transfer_type),
		      GINT_TO_POINTER (AUTO));
    bin =
	gtk_item_factory_get_widget (item_factory,
				     "<main>/Utilities/Transfer type/Binary");
    gtk_radio_menu_item_set_group (GTK_RADIO_MENU_ITEM (bin),
				   gtk_radio_menu_item_group
				   (GTK_RADIO_MENU_ITEM (aut)));
    g_signal_connect (GTK_OBJECT (bin), "toggled",
		      G_CALLBACK (_gui_set_transfer_type),
		      GINT_TO_POINTER (BINARY));
    ascii =
	gtk_item_factory_get_widget (item_factory,
				     "<main>/Utilities/Transfer type/ASCII");
    gtk_radio_menu_item_set_group (GTK_RADIO_MENU_ITEM (ascii),
				   gtk_radio_menu_item_group
				   (GTK_RADIO_MENU_ITEM (bin)));
    g_signal_connect (GTK_OBJECT (ascii), "toggled",
		      G_CALLBACK (_gui_set_transfer_type),
		      GINT_TO_POINTER (ASCII));
    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (bin), FALSE);
    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ascii), FALSE);
    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (aut), TRUE);
    ttype = AUTO;

    if (menubar)
	/*
	 * Finally, return the actual menu bar created by the item factory. 
	 */
	*menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}

static void
_gui_set_transfer_type (GtkWidget * w, gpointer data)
{
    /*
     * this only sets everything up. We need the file transfer to be 
     * automatic to get dir listings, etc, so we force the transfer type
     * only when needed 
     */
    ttype = GPOINTER_TO_INT (data);
    switch (ttype) {
    case AUTO:
	status_push ("Transfer file type set to automatic");
	break;
    case BINARY:
	status_push ("Transfer file type set to Binary");
	break;
    case ASCII:
	status_push ("Transfer file type set to ASCII");
	break;
    }
}

/*
 * Enables all menu items that need to be enabled when connection to HP
 * Calculator is established 
 */
void
gui_set_menu_connected ()
{
    GtkWidget          *tw;
    const char         *t;
    int                 i = 0;

    /*
     * make menu correct 
     */
    t = on_items[0];
    while (t != NULL) {
	tw = gtk_item_factory_get_widget (item_factory, t);
	gtk_widget_set_sensitive (tw, TRUE);
	t = on_items[++i];
    }

    t = off_items[0];
    i = 0;
    while (t != NULL) {
	tw = gtk_item_factory_get_widget (item_factory, t);
	gtk_widget_set_sensitive (tw, FALSE);
	t = off_items[++i];
    }

    if (hp_calculator == HP49G) {
	t = hp49_items[0];
	i = 0;
	while (t != NULL) {
	    tw = gtk_item_factory_get_widget (item_factory, t);
	    gtk_widget_set_sensitive (tw, TRUE);
	    t = hp49_items[++i];
	}
    }

    /*
     * now pop_menu 
     */
    gui_get_pop_menu (NULL);
}

/*
 * Disables all menu items that need to be disabled when connection to HP
 * Calculator is down 
 */
void
gui_set_menu_disconnected ()
{
    GtkWidget          *tw;
    const char         *t;
    int                 i = 0;

    /*
     * make menu correct 
     */
    t = off_items[0];
    while (t != NULL) {
	tw = gtk_item_factory_get_widget (item_factory, t);
	gtk_widget_set_sensitive (tw, TRUE);
	t = off_items[++i];
    }
    t = on_items[0];
    i = 0;
    while (t != NULL) {
	tw = gtk_item_factory_get_widget (item_factory, t);
	gtk_widget_set_sensitive (tw, FALSE);
	t = on_items[++i];
    }
    t = hp49_items[0];
    i = 0;
    while (t != NULL) {
	tw = gtk_item_factory_get_widget (item_factory, t);
	gtk_widget_set_sensitive (tw, FALSE);
	t = hp49_items[++i];
    }

    /*
     * now pop_menu 
     */
    gui_get_pop_menu (NULL);
}

void
gui_get_pop_menu (GtkWidget * popmenu)
{
    static int          loaded = 0;
    static GtkWidget   *transfer_item, *move_item;
    GtkWidget          *menu_items;

    if (loaded == 0) {
	transfer_item = gtk_menu_item_new_with_label ("Transfer");
	gtk_menu_append (GTK_MENU (popmenu), transfer_item);
	g_signal_connect (GTK_OBJECT (transfer_item), "activate",
			  G_CALLBACK (io_transfer), GINT_TO_POINTER (TRUE));
	gtk_widget_set_sensitive (transfer_item, FALSE);
	gtk_widget_show (transfer_item);

	move_item = gtk_menu_item_new_with_label ("Move");
	gtk_menu_append (GTK_MENU (popmenu), move_item);
	g_signal_connect (GTK_OBJECT (move_item), "activate",
			  G_CALLBACK (io_move), NULL);
	gtk_widget_set_sensitive (move_item, FALSE);
	gtk_widget_show (move_item);

	menu_items = gtk_menu_item_new ();
	gtk_menu_append (GTK_MENU (popmenu), menu_items);
	gtk_widget_show (menu_items);

	menu_items = gtk_menu_item_new_with_label ("Edit");
	gtk_menu_append (GTK_MENU (popmenu), menu_items);
	g_signal_connect (GTK_OBJECT (menu_items), "activate",
			  G_CALLBACK (io_edit), NULL);
	gtk_widget_show (menu_items);

	menu_items = gtk_menu_item_new_with_label ("Copy...");
	gtk_menu_append (GTK_MENU (popmenu), menu_items);
	g_signal_connect (GTK_OBJECT (menu_items), "activate",
			  G_CALLBACK (io_copy), NULL);
	gtk_widget_show (menu_items);

	menu_items = gtk_menu_item_new_with_label ("Rename...");
	gtk_menu_append (GTK_MENU (popmenu), menu_items);
	g_signal_connect (GTK_OBJECT (menu_items), "activate",
			  G_CALLBACK (io_renamew), NULL);
	gtk_widget_show (menu_items);

	menu_items = gtk_menu_item_new_with_label ("Unpack");
	gtk_menu_append (GTK_MENU (popmenu), menu_items);
	g_signal_connect (GTK_OBJECT (menu_items), "activate",
			  G_CALLBACK (io_unpack), NULL);
	gtk_widget_show (menu_items);

	menu_items = gtk_menu_item_new_with_label ("Delete");
	gtk_menu_append (GTK_MENU (popmenu), menu_items);
	g_signal_connect (GTK_OBJECT (menu_items), "activate",
			  G_CALLBACK (io_delete), GINT_TO_POINTER (TRUE));
	gtk_widget_show (menu_items);

	menu_items = gtk_menu_item_new ();
	gtk_menu_append (GTK_MENU (popmenu), menu_items);
	gtk_widget_show (menu_items);

	menu_items = gtk_menu_item_new_with_label ("Create Directory ...");
	gtk_menu_append (GTK_MENU (popmenu), menu_items);
	g_signal_connect (GTK_OBJECT (menu_items), "activate",
			  G_CALLBACK (io_create_dir), NULL);
	gtk_widget_show (menu_items);
	loaded = 1;
    } else if (connected == ONLINE) {
	gtk_widget_set_sensitive (move_item, TRUE);
	gtk_widget_set_sensitive (transfer_item, TRUE);
    } else {
	gtk_widget_set_sensitive (move_item, FALSE);
	gtk_widget_set_sensitive (transfer_item, FALSE);
    }
}

/*******************************************************/

/*******************************************************/

/*******************                 *******************/

/******************* Transfer Dialog *******************/

/*******************                 *******************/

/*******************************************************/

/*******************************************************/

/*
 * This is the callback function used for displaying the transfer statistics. 
 * Its address is passed to k_read_file_transfer() and
 * k_read_server_transfer() 
 */
void
gui_transfer_callback (unsigned int *bytes, unsigned int *p,
		       unsigned int *cps, void *data)
{
    GtkAdjustment      *adj;
    char                status[BUFFERSIZE];

    while (gtk_events_pending ())
	gtk_main_iteration ();

    /*
     * Update progress bar 
     */
    adj = GTK_PROGRESS (((callback_data *) data)->progressbar)->adjustment;
    /*
     * Hack: Assume 100 is used to indicate a active mode progress bar.
     * However, should a file have the size of 100, this will fail 
     */
    if (adj->upper == 100) {
	gfloat              new_val;

	new_val =
	    gtk_progress_get_value (GTK_PROGRESS
				    (((callback_data *) data)->progressbar)) +
	    1;
	if (new_val == 100)
	    new_val = 0;
	gtk_progress_set_value (GTK_PROGRESS
				(((callback_data *) data)->progressbar),
				new_val);
    } else {
	gtk_progress_set_show_text (GTK_PROGRESS
				    (((callback_data *) data)->progressbar),
				    TRUE);
	gtk_progress_set_format_string (GTK_PROGRESS
					(((callback_data *) data)->
					 progressbar), "Completed %p%%");
	gtk_progress_set_value (GTK_PROGRESS
				(((callback_data *) data)->progressbar),
				(gfloat) * bytes);
    }

    /*
     * Update info label 
     */
    sprintf (status, "%d bytes sent\n%d CPS", *bytes, *cps);
    gtk_label_set_text (GTK_LABEL (((callback_data *) data)->infolabel),
			status);
    while (gtk_events_pending ())
	gtk_main_iteration ();

}

/*
 * Function expects in data a pointer to an int, which will be set to 1 to
 * flag the cancelation of the transfer 
 */
static void
_gui_cancel_transfer (GtkWidget * w, gpointer data)
{
    hp_cancel_transfer ();
    *((int *) data) = 1;
}

/*
 * Creates a transfer dialog used by transfer(), backup() and restore(). The
 * caller is responsible for destroying the widgets afterwards. 
 */
void
gui_transfer_dialog_create (GtkWidget ** dialog, GtkWidget ** filelabel,
			    GtkWidget ** infolabel, GtkWidget ** progressbar,
			    int *cancel_flag)
{
    GtkAdjustment      *adj;
    GtkWidget          *button;
    GtkWidget          *tmp;

    if ((dialog == NULL) | (filelabel == NULL) | (infolabel == NULL))
	return;

    *dialog = gtk_dialog_new ();
    gtk_window_set_title (GTK_WINDOW (*dialog), "Uploading...");
    gtk_window_set_default_size (GTK_WINDOW (*dialog), 200, 250);
    gtk_container_set_border_width (GTK_CONTAINER (*dialog), 5);
    gtk_window_set_modal (GTK_WINDOW (*dialog), TRUE);

    /*
     * filename
     */
    tmp = gtk_label_new ("Transferring:");
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (*dialog)->vbox),
			tmp, FALSE, FALSE, 10);
    gtk_widget_show (tmp);
    *filelabel = gtk_label_new ("file");
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (*dialog)->vbox),
			*filelabel, FALSE, FALSE, 10);
    gtk_widget_show (*filelabel);

    /*
     * Progress bar. The values of the GtkAdjustment adj, have to be set by
     * the caller after this function returns
     */
    adj = (GtkAdjustment *) gtk_adjustment_new (0, 0, 0, 0, 0, 0);
    *progressbar = gtk_progress_bar_new_with_adjustment (adj);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (*dialog)->vbox), *progressbar,
			FALSE, FALSE, 10);
    gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (*progressbar),
				      GTK_PROGRESS_LEFT_TO_RIGHT);

    gtk_widget_show (*progressbar);
    /*
     * show status
     */
    *infolabel = gtk_label_new ("0 bytes sent\n0 CPS");
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (*dialog)->vbox),
			*infolabel, FALSE, FALSE, 10);
    gtk_widget_show (*infolabel);

    button = gtk_button_new_with_label ("Cancel");
    set_button_default_size (button);
    g_signal_connect (GTK_OBJECT (button), "clicked",
		      G_CALLBACK (_gui_cancel_transfer), cancel_flag);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_widget_destroy),
			       GTK_OBJECT (*dialog));
    gtk_box_pack_end (GTK_BOX (GTK_DIALOG (*dialog)->action_area),
		      button, FALSE, FALSE, 0);
    gtk_widget_show (button);
    gtk_widget_show (*dialog);
}

void
gui_transfer_progress_bar_set_activity (GtkProgressBar * pb)
{
    /*
     * Use activity mode for the progress bar 
     */
    gtk_progress_configure (GTK_PROGRESS (pb), 0, 0, 100);
    gtk_progress_set_activity_mode (GTK_PROGRESS (pb), TRUE);
    gtk_progress_bar_set_activity_step (pb, 5);
    gtk_progress_bar_set_activity_blocks (pb, 5);
}

/*****************************************************/

/*****************************************************/

/*******************               *******************/

/******************* Backup Dialog *******************/

/*******************               *******************/

/*****************************************************/

/*****************************************************/

/*
 * Function sets data to OK. Data has to be a pointer to a RETVAL type 
 */
static void
_gui_backup_ok (GtkWidget * w, gpointer data)
{
    *((RETVALUE *) data) = OK;
    gtk_main_quit ();
}

/*
 * Function sets data to CANCEL. Data has to be a pointer to a RETVAL type 
 */
static void
_gui_backup_cancel (GtkWidget * w, gpointer data)
{
    hp_cancel_transfer ();
    *((RETVALUE *) data) = CANCEL;
    gtk_main_quit ();
}

GtkWidget          *
gui_backup_dialog_create (RETVALUE * ret_val)
{
    GtkWidget          *w;

    w = gtk_file_selection_new ("Select File");
    gtk_window_set_modal (GTK_WINDOW (w), TRUE);

    /*
     * Handlers for the OK Button
     */
    g_signal_connect (GTK_OBJECT
		      (GTK_FILE_SELECTION (w)->ok_button),
		      "clicked", G_CALLBACK (_gui_backup_ok), ret_val);

    /*
     * Handlers for the Cancel Button 
     */
    g_signal_connect (GTK_OBJECT
		      (GTK_FILE_SELECTION (w)->cancel_button),
		      "clicked", G_CALLBACK (_gui_backup_cancel), ret_val);

    gtk_widget_show (w);
    gtk_main ();
    gtk_widget_hide (w);
    return w;
}

/******************************************************/

/******************************************************/

/*******************                *******************/

/******************* Command Window *******************/

/*******************                *******************/

/******************************************************/

/******************************************************/

static void
_gui_quit_command_window (GtkWidget * w, gpointer data)
{
    gtk_widget_destroy (GTK_WIDGET (data));
    /*
     * Reset the flag to state that there is no Command Window 
     */
    command_window_created = FALSE;
}

static void
_gui_command_window_prepare_args (GtkWidget * w, gpointer data)
{
    CWIA_P (data)->args->command =
	gtk_entry_get_text (GTK_ENTRY (CWIA_P (data)->input));
}

static void
_gui_clear_input_entry (GtkWidget * w, gpointer data)
{
    gtk_entry_set_text (GTK_ENTRY (data), "");
}

/*
 * Creates the Command Window. The callback function send_command_handler
 * gets passed a command_window_send_args struct as data.
 */
GtkWidget          *
gui_command_window_create (GCallback send_command_handler)
{
    GtkWidget          *command_window;
    GtkWidget          *spacer_hbox;
    GtkWidget          *vbox1;
    GtkWidget          *vbox2;
    GtkWidget          *output_frame;
    GtkWidget          *output_box;
    GtkWidget          *output;
    GtkTextBuffer      *output_buffer;
    GtkWidget          *input_frame;
    GtkWidget          *input;
    GtkWidget          *vscrollwin;
    GtkWidget          *sep;
    GtkWidget          *button_box;
    GtkWidget          *send_button;
    GtkWidget          *quit_button;
    static command_window_send_args args;
    static command_window_internal_args internal_args;

    /*
     * We don't want to create several Command Windows 
     */
    if (command_window_created == TRUE)
	return NULL;
    /*
     * Create the command window 
     */
    command_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_default_size (GTK_WINDOW (command_window), 300, 200);
    gtk_window_set_title (GTK_WINDOW (command_window), "Command");

    /*
     * Create the horizontal hbox used for spacing. If somebody got a better
     * idea, lemme know 
     */
    spacer_hbox = gtk_hbox_new (TRUE, 0);
    gtk_container_add (GTK_CONTAINER (command_window), spacer_hbox);

    /*
     * Create the vbox for the Buttons and entry widgets
     */
    vbox1 = gtk_vbox_new (FALSE, 5);
    gtk_box_pack_start (GTK_BOX (spacer_hbox), vbox1, TRUE, TRUE, 5);

    /*
     * Create the vbox for the input and output widgets 
     */
    vbox2 = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, FALSE, 2);

    /*
     * Create the output frame 
     */
    output_frame = gtk_frame_new ("Output");
    gtk_box_pack_start (GTK_BOX (vbox2), output_frame, TRUE, TRUE, 2);

    /*
     * Create a box which is used for scrolling for the output 
     */
    output_box = gtk_hbox_new (FALSE, 5);
    gtk_container_add (GTK_CONTAINER (output_frame), output_box);

    /*
     * Create the scrolled window for the text view
     */
    vscrollwin = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (vscrollwin),
				    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    gtk_container_set_border_width (GTK_CONTAINER (vscrollwin), 2);
    gtk_widget_set_size_request (GTK_WIDGET (vscrollwin), 100, 200);
    gtk_box_pack_start (GTK_BOX (output_box), vscrollwin, TRUE, TRUE, 0);

    /*
     * Create the output text (Changed to cope with GTK 2.x)
     */
    output = gtk_text_view_new ();
    output_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (output));
    gtk_text_view_set_editable (GTK_TEXT_VIEW (output), FALSE);
    gtk_container_add (GTK_CONTAINER (vscrollwin), output);

    /*
     * Create the input frame 
     */
    input_frame = gtk_frame_new ("Command");
    gtk_box_pack_start (GTK_BOX (vbox2), input_frame, TRUE, FALSE, 2);

    /*
     * Create the input entry 
     */
    input = gtk_entry_new ();
    gtk_widget_grab_focus (input);
    gtk_container_add (GTK_CONTAINER (input_frame), input);

    /*
     * Seperator 
     */
    sep = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (vbox1), sep, TRUE, FALSE, 2);

    /*
     * Create the button box and add to vbox1 
     */
    button_box = gtk_hbutton_box_new ();
    gtk_box_pack_start (GTK_BOX (vbox1), button_box, FALSE, FALSE, 2);

    /*
     * Add send button to the button box 
     */
    send_button = gtk_button_new_with_label ("Send");
    set_button_default_size (send_button);
    gtk_box_pack_start (GTK_BOX (button_box), send_button, FALSE, FALSE, 2);

    /*
     * Add quit button to the button box 
     */
    quit_button = gtk_button_new_with_label ("Quit");
    set_button_default_size (quit_button);
    gtk_box_pack_start (GTK_BOX (button_box), quit_button, FALSE, FALSE, 2);

    /*
     * Add signal handler to quit button 
     */
    g_signal_connect (GTK_OBJECT (quit_button), "clicked",
		      G_CALLBACK (_gui_quit_command_window), command_window);

    /*
     * Prepare arguments for the send button signal handler
     * _gui_command_window_prepare_args 
     */
    internal_args.input = input;	/* The widget holding the input */
    internal_args.args = &args;	/* The arguments passed later to the user * * 
				 * defined callback function */

    /*
     * Prepare arguments for send button signal handler send_command_handler provided by the user
     */
    args.command = NULL;	/* This value will be filled by * *
				 * _gui_command_window_prepare_args */
    args.output = output;

    /*
     * Add signal handler to send button 
     */
    g_signal_connect (GTK_OBJECT (send_button), "clicked",
		      G_CALLBACK (_gui_command_window_prepare_args),
		      &internal_args);
    g_signal_connect (GTK_OBJECT (send_button), "clicked",
		      send_command_handler, &args);
    g_signal_connect (GTK_OBJECT (send_button), "clicked",
		      G_CALLBACK (_gui_clear_input_entry), input);

    gtk_widget_show_all (command_window);

    /*
     * Set the flag to indicate the Command Window is created, and no new
     * Command Window should be created by further calls to
     * gui_command_window_create(). 
     */
    command_window_created = TRUE;
    return command_window;
}

/*************************************************/

/*************************************************/

/*******************           *******************/

/******************* HowTo Use *******************/

/*******************           *******************/

/*************************************************/

/*************************************************/
void
gui_howtouse (GtkWidget * w, gpointer data)
{
    message_box (MSG_INF_HOWTO_USE_TITLE, MSG_INF_HOWTO_USE);
}

/*********************************************/

/*********************************************/

/*******************       *******************/

/******************* ABOUT *******************/

/*******************       *******************/

/*********************************************/

/********************************************/
void
gui_about (GtkWidget * w, gpointer data)
{
    message_box (MSG_INF_ABOUT_TITLE, MSG_INF_ABOUT);
}

/*********************************************************/

/*********************************************************/

/*******************                   *******************/

/******************* Message Functions *******************/

/*******************                   *******************/

/*********************************************************/

/*********************************************************/

/*
 * Prints a message according the k_status variable s. Depending on the
 * severity it is either a status_push() or message_box() command used to
 * display the message. 
 */
void
gui_print_nice_kstatus (k_status s)
{
    switch (s) {
    case K_OK:
	status_push (MSG_INF_K_OK);
	break;
    case K_CONNECTED:
	status_push (MSG_INF_K_CONNECTED);
	break;
    case K_NOTCONNECTED:
	status_push (MSG_INF_K_NOTCONNECTED);
	break;
    case K_FORKPTY:
	message_box (MSG_FAT, MSG_FAT_K_FORKPTY);
	break;
    case K_EXECKERMIT:
	message_box (MSG_FAT, MSG_FAT_K_EXECKERMIT);
	break;
    case K_FCNTL:
	message_box (MSG_FAT, MSG_FAT_K_FCNTL);
	break;
    case K_PIPE:
	message_box (MSG_FAT, MSG_FAT_K_PIPE);
	break;
    case K_INITKERM:
	message_box (MSG_ERR, MSG_ERR_K_INITKERM);
	break;
    case K_MALLOC:
	message_box (MSG_FAT, MSG_FAT_MALLOC);
	break;
    case K_INVARG:
	message_box (MSG_ERR, MSG_ERR_K_INVARG);
	break;
    case K_WRITE:
	message_box (MSG_FAT, MSG_FAT_WRITE);
	break;
    case K_READ:
	message_box (MSG_FAT, MSG_FAT_READ);
	break;
    case K_EOF:
	status_push (MSG_INF_EOF);
	break;
    case K_TIMEOUT:
	status_push (MSG_WARN_TIMEOUT);
	break;
    case K_SELECT:
	message_box (MSG_FAT, MSG_FAT_SELECT);
	break;
    case K_STREAM:
	message_box (MSG_FAT, MSG_FAT_STREAM);
	break;
    case K_CANCEL:
	status_push (MSG_INF_CANCEL);
	break;
    case K_BUFFER:
	message_box (MSG_ERR, MSG_ERR_BUFFER);
	break;
    case K_UNKNOWN:
	message_box (MSG_FAT, MSG_FAT_UNKNOWN);
	break;
    default:
	message_box (MSG_FAT, MSG_FAT_UNHANDLED);
	break;
    }
}

/*
 * Prints a message according the hp_status variable s. Depending on the
 * severity it is either a status_push() or message_box() command used to
 * display the message. 
 */
void
gui_print_nice_hpstatus (hp_status s)
{
    if (s == HP_KERMIT) {
	gui_print_nice_kstatus (k_errno);
	return;
    }

    switch (s) {
    case HP_OK:
	status_push (MSG_INF_HP_OK);
	break;
    case HP_INVARG:
	message_box (MSG_ERR, MSG_ERR_HP_INVARG);
	break;
    case HP_DIR:
	message_box (MSG_ERR, MSG_ERR_HP_DIR);
	break;
    case HP_MALLOC:
	message_box (MSG_FAT, MSG_FAT_MALLOC);
	break;
    case HP_PARSE:
	message_box (MSG_ERR, MSG_ERR_HP_PARSE);
	break;
    case HP_BUFFER:
	message_box (MSG_ERR, MSG_ERR_BUFFER);
	break;
    case HP_FOPEN:
	message_box (MSG_FAT, MSG_FAT_FOPEN);
	break;
    case HP_FREAD:
	message_box (MSG_FAT, MSG_FAT_FREAD);
	break;
    default:
	message_box (MSG_FAT, MSG_FAT_UNHANDLED);
	break;
    }
}

/*******************************************************/

/*******************************************************/

/*******************                 *******************/

/******************* Check Functions *******************/

/*******************                 *******************/

/*******************************************************/

/*******************************************************/

/*
 * Function returns TRUE if the user selected a directory in either the
 * directory clist or the files clist 
 */
gboolean
gui_is_directory_selected ()
{
    if (whereami_h == FILES) {
	char               *rowdata;

	/*
	 * Retrieve the row data for the right clist 
	 */
	if (whereami_v == PC) {
	    rowdata =
		_gui_file_list_get_type_str_selected (GTK_TREE_VIEW
						      (files_pc));
	} else {
	    rowdata =
		_gui_file_list_get_type_str_selected (GTK_TREE_VIEW
						      (files_hp));
	}
	if (rowdata == NULL)
	    return FALSE;

	if (strcmp (ROW_DATA_DIRECTORY, rowdata) == 0)
	    return TRUE;
    } else {
	return TRUE;
    }
    return FALSE;
}

/*
 * Function returns TRUE if the user selected a file in the files clist 
 */
gboolean
gui_is_file_selected ()
{
    if (whereami_h == DIRS) {
	return FALSE;
    } else {
	char               *rowdata;

	if (whereami_v == PC) {
	    rowdata =
		_gui_file_list_get_type_str_selected (GTK_TREE_VIEW
						      (files_pc));
	} else {
	    rowdata =
		_gui_file_list_get_type_str_selected (GTK_TREE_VIEW
						      (files_hp));
	}
	if (strcmp (rowdata, ROW_DATA_DIRECTORY) != 0)
	    return TRUE;
	else
	    return FALSE;
    }

    return FALSE;
}

/***********************************************************************/

/***********************************************************************/

/*******************                                 *******************/

/******************* Lists for Directories and Files *******************/

/*******************                                 *******************/

/***********************************************************************/

/***********************************************************************/
void
gui_directory_list_create (GtkWidget ** list, GtkWidget ** scroll_win,
			   VPANED what)
{
    GtkListStore       *store;
    GtkCellRenderer    *renderer;
    GtkTreeViewColumn  *column;
    GtkTreeSelection   *select;

    store = gtk_list_store_new (DLC_NUMBER, G_TYPE_STRING);
    *list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));

    /*
     * Set the signal handler for selecting items in the list 
     */
    select = gtk_tree_view_get_selection (GTK_TREE_VIEW (*list));
    gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
    g_signal_connect (G_OBJECT (*list),
		      "button_release_event",
		      G_CALLBACK (_gui_dir_select_row),
		      GINT_TO_POINTER (what));
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Not Connected",
						       renderer,
						       "text", DLC_NAME,
						       NULL);
    gtk_tree_view_column_set_clickable (column, TRUE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (*list), column);
    g_signal_connect (G_OBJECT (column), "clicked",
		      G_CALLBACK (io_choose_dir), GINT_TO_POINTER (what));

    gtk_widget_set_usize (GTK_WIDGET (*list), 200, 200);

    /*
     * Create the scrolled window 
     */
    *scroll_win = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (*scroll_win), *list);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (*scroll_win),
				    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    gtk_container_set_border_width (GTK_CONTAINER (*scroll_win), 0);

    gtk_widget_show (*list);
    gtk_widget_show (*scroll_win);
}

/*
 * callback function to deal with lists 
 */
static              gint
_gui_dir_select_row (GtkWidget * widget, GdkEventButton * event,
		     gpointer which)
{
    GtkTreeView        *list;
    struct data        *d;
    gchar              *directory;

    /*
     * Set globally where we are, either in the PC or the HP Panel 
     */
    whereami_v = (VPANED) GPOINTER_TO_INT (which);

    /*
     * Set globally that we are in a directory clist 
     */
    whereami_h = DIRS;

    /*
     * Get the data according the vertical panel we are in 
     */
    d = whereami_v == PC ? &data_pc : &data_hp;

    list = GTK_TREE_VIEW ((whereami_v == HP ? dir_hp : dir_pc));

    directory = _gui_dir_list_get_name_selected (list);
    if (directory == NULL)
	return FALSE;
    g_free (d->dir);
    d->dir = g_strdup (directory);

    io_change_dir (NULL, NULL);

    return FALSE;

}

static gchar       *
_gui_dir_list_get_name_selected (GtkTreeView * list)
{
    GtkTreeSelection   *list_selection;
    GtkTreeModel       *list_model;
    GList              *selected_rows;
    GtkTreeIter         list_iter;
    gboolean            got_iter;
    gchar              *tmp;

    /*
     * Retrieve the selection 
     */
    list_selection = gtk_tree_view_get_selection (list);

    /*
     * Retrieve all selected rows 
     */
    selected_rows = gtk_tree_selection_get_selected_rows
	(list_selection, &list_model);
    if ((selected_rows == NULL) || (selected_rows->data == NULL))
	return NULL;

    /*
     * Where taking only the first entry of the list, because we want to safe 
     * a file name into the data struct and not the entire list. Sorta hack. 
     */
    got_iter =
	gtk_tree_model_get_iter (list_model, &list_iter,
				 (GtkTreePath *) selected_rows->data);
    if (got_iter != TRUE)
	return NULL;

    gtk_tree_model_get (list_model, &list_iter, DLC_NAME, &tmp, -1);

    g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
    g_list_free (selected_rows);

    return tmp;
}

void
gui_file_list_create (GtkWidget ** list, GtkWidget ** scroll_win, VPANED what)
{
    GtkListStore       *store;
    GtkCellRenderer    *renderer;
    GtkTreeViewColumn  *column;
    GtkTreeSelection   *select;

    store =
	gtk_list_store_new (FLC_NUMBER, G_TYPE_STRING, GDK_TYPE_PIXBUF,
			    G_TYPE_STRING, G_TYPE_STRING);
    *list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));

    /*
     * Set the signal handler for selecting items in the list 
     */
    select = gtk_tree_view_get_selection (GTK_TREE_VIEW (*list));
    gtk_tree_selection_set_mode (select, GTK_SELECTION_MULTIPLE);

    /*
     * The type column 
     */
    renderer = gtk_cell_renderer_pixbuf_new ();
    column = gtk_tree_view_column_new_with_attributes ("Type",
						       renderer,
						       "pixbuf", FLC_TYPE,
						       NULL);
    /*
     * Sizing of the Type Column 
     */
    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
    gtk_tree_view_column_set_min_width (column, 50);
    gtk_tree_view_column_set_resizable (column, FALSE);

    /*
     * Sorting of the Type Column 
     */
    gtk_tree_view_column_set_sort_column_id (column, FLC_TYPESTR);
    gtk_tree_view_column_set_sort_indicator (column, TRUE);

    gtk_tree_view_append_column (GTK_TREE_VIEW (*list), column);

    /*
     * The Name column 
     */
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Name",
						       renderer,
						       "text", FLC_NAME,
						       NULL);
    /*
     * Sizing of the Name Column 
     */
    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
    gtk_tree_view_column_set_min_width (column, 250);
    gtk_tree_view_column_set_resizable (column, TRUE);

    /*
     * Sorting of the Name Column 
     */
    gtk_tree_view_column_set_sort_column_id (column, FLC_NAME);
    gtk_tree_view_column_set_sort_indicator (column, TRUE);

    gtk_tree_view_append_column (GTK_TREE_VIEW (*list), column);

    /*
     * The Size column 
     */
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Size",
						       renderer,
						       "text", FLC_SIZE,
						       NULL);
    /*
     * Sizing of the Size Column 
     */
    gtk_tree_view_column_set_sort_column_id (column, FLC_SIZE);
    gtk_tree_view_column_set_sort_indicator (column, TRUE);
    gtk_tree_view_column_set_resizable (column, TRUE);

    /*
     * Sorting of the Size Column 
     */
    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);

    gtk_tree_view_append_column (GTK_TREE_VIEW (*list), column);

    /*
     * Make the Name Column Searchable 
     */
    gtk_tree_view_set_search_column (GTK_TREE_VIEW (*list), FLC_NAME);

    /*
     * Initialy sort by name 
     */
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
					  FLC_NAME, GTK_SORT_ASCENDING);

    g_signal_connect (G_OBJECT (*list),
		      "button_press_event",
		      G_CALLBACK (_gui_file_select_row),
		      GINT_TO_POINTER (what));

    g_signal_connect (G_OBJECT (*list), "button_release_event",
		      G_CALLBACK (_gui_file_select_row),
		      GINT_TO_POINTER (what));

    g_signal_connect_swapped (G_OBJECT (*list), "event",
			      G_CALLBACK (_gui_pop_menu),
			      GTK_OBJECT (pop_up_menu));

    /*
     * Create scroll window 
     */
    *scroll_win = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (*scroll_win), *list);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (*scroll_win),
				    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    gtk_container_set_border_width (GTK_CONTAINER (*scroll_win), 0);
    gtk_widget_show (*list);
    gtk_widget_show (*scroll_win);

}

static              gint
_gui_file_select_row (GtkWidget * widget, GdkEventButton * event,
		      gpointer which)
{
    GtkTreeView        *list;
    struct data        *d;
    gchar              *name;
    gchar              *type_str;

    /*
     * Set globally where we are, either in the PC or HP panel 
     */
    whereami_v = (VPANED) GPOINTER_TO_INT (which);

    /*
     * Set globally that we are in a files list 
     */
    whereami_h = FILES;

    /*
     * Get the data according the panel we are in 
     */
    d = whereami_v == PC ? &data_pc : &data_hp;

    list = GTK_TREE_VIEW ((whereami_v == HP ? files_hp : files_pc));

    /*
     * Set the selected file name 
     */
    name = _gui_file_list_get_name_selected (list);
    if (name == NULL)
	return FALSE;
    g_free (d->filename);
    d->filename = g_strdup (name);

    /*
     * Get the type of the selected file 
     */
    type_str = _gui_file_list_get_type_str_selected (list);
    if (type_str == NULL)
	return FALSE;
    g_free (d->file_type_str);
    d->file_type_str = g_strdup (type_str);

    if (d->filename != NULL && event != NULL) {
	if (event->type == GDK_2BUTTON_PRESS) {
	    char               *t;

	    t = _gui_file_list_get_type_str_selected (list);

	    if (strstr (t, ROW_DATA_DIRECTORY))
		io_change_dir (NULL, NULL);
	    else if (connected == ONLINE)
		io_transfer (NULL, GINT_TO_POINTER (TRUE));
	    else
		io_edit (NULL, NULL);
	}

    }
    return FALSE;
}

static gchar       *
_gui_file_list_get_type_str_selected (GtkTreeView * list)
{
    GtkTreeSelection   *list_selection;
    GtkTreeModel       *list_model;
    GList              *selected_rows;
    GtkTreeIter         list_iter;
    gboolean            got_iter;
    gchar              *tmp;

    /*
     * Retrieve the selection 
     */
    list_selection = gtk_tree_view_get_selection (list);

    /*
     * Retrieve all selected rows 
     */
    selected_rows = gtk_tree_selection_get_selected_rows
	(list_selection, &list_model);
    if ((selected_rows == NULL) || (selected_rows->data == NULL))
	return NULL;

    /*
     * Where taking only the first entry of the list, because we want to safe 
     * a file name into the data struct and not the entire list. Sorta hack. 
     */
    got_iter =
	gtk_tree_model_get_iter (list_model, &list_iter,
				 (GtkTreePath *) selected_rows->data);
    if (got_iter != TRUE)
	return NULL;

    gtk_tree_model_get (list_model, &list_iter, FLC_TYPESTR, &tmp, -1);

    g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
    g_list_free (selected_rows);

    return tmp;
}

static gchar       *
_gui_file_list_get_name_selected (GtkTreeView * list)
{
    GtkTreeSelection   *list_selection;
    GtkTreeModel       *list_model;
    GList              *selected_rows;
    GtkTreeIter         list_iter;
    gboolean            got_iter;
    gchar              *tmp;

    /*
     * Retrieve the selection 
     */
    list_selection = gtk_tree_view_get_selection (list);

    /*
     * Retrieve all selected rows 
     */
    selected_rows = gtk_tree_selection_get_selected_rows
	(list_selection, &list_model);
    if ((selected_rows == NULL) || (selected_rows->data == NULL))
	return NULL;

    /*
     * Where taking only the first entry of the list, because we want to safe 
     * a file name into the data struct and not the entire list. Sorta hack. 
     */
    got_iter =
	gtk_tree_model_get_iter (list_model, &list_iter,
				 (GtkTreePath *) selected_rows->data);
    if (got_iter != TRUE)
	return NULL;

    gtk_tree_model_get (list_model, &list_iter, FLC_NAME, &tmp, -1);

    g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
    g_list_free (selected_rows);

    return tmp;
}

gboolean
gui_file_list_single_selection (GtkTreeView * list)
{
    GtkTreeSelection   *list_selection;

    if (list == NULL)
	return FALSE;

    list_selection = gtk_tree_view_get_selection (list);

    return (gtk_tree_selection_count_selected_rows (list_selection) >
	    1 ? FALSE : TRUE);
}

gchar              *
gui_file_list_get_name_by_iter (GtkTreeModel * model, GtkTreeIter * iter)
{
    gchar              *tmp;

    if ((model == NULL) || (iter == NULL))
	return NULL;

    gtk_tree_model_get (model, iter, FLC_NAME, &tmp, -1);
    return tmp;
}

gchar              *
gui_file_list_get_type_str_by_iter (GtkTreeModel * model, GtkTreeIter * iter)
{
    gchar              *tmp;

    if ((model == NULL) || (iter == NULL))
	return NULL;

    gtk_tree_model_get (model, iter, FLC_TYPESTR, &tmp, -1);
    return tmp;
}

gchar              *
gui_file_list_get_size_str_by_iter (GtkTreeModel * model, GtkTreeIter * iter)
{
    gchar              *tmp;

    if ((model == NULL) || (iter == NULL))
	return NULL;

    gtk_tree_model_get (model, iter, FLC_SIZE, &tmp, -1);
    return tmp;
}

void
gui_select_all_file_list_item (GtkWidget * w, gpointer wdc	/* we don't
								 * care */ )
{
    GtkTreeView        *list;
    GtkTreeSelection   *selection;

    list = GTK_TREE_VIEW (whereami_v == PC ? files_pc : files_hp);
    selection = gtk_tree_view_get_selection (list);
    gtk_tree_selection_select_all (selection);
}

void
gui_unselect_all_file_list_item (GtkWidget * w, gpointer wdc	/* we don't * 
								 * care */ )
{
    GtkTreeView        *list;
    GtkTreeSelection   *selection;

    list = GTK_TREE_VIEW (whereami_v == PC ? files_pc : files_hp);
    selection = gtk_tree_view_get_selection (list);
    gtk_tree_selection_unselect_all (selection);
}
