
/*
 * HPTalx
 * A PC<->HP48/49 communication program for Linux
 *
 * (C) 1999 by Bruno Barberi Gnecco
 * (C) 2005 by Rafael Ostertag
 *
 * $Id: gui_utils.c,v 1.3 2007/09/29 21:21:01 guengel Exp $
 *
 *
 * Copyright note:
 * 
 * This program is free software; you can
 * redistribute it and/or modify it under the
 * terms of the GNU General Public License as
 * published by the Free Software Foundation;
 * either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that
 * it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU
 * General Public License along with this
 * program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gui_utils.h"
#include "messages.h"

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

GtkWidget          *status = NULL;	/* Status bar */
static gint         status_bar_ctx_id = 0;
static guint        status_bar_stack_height = 0;

/*
 * Used by entry pop-up 
 */
struct entry
{
    char               *text;
    GtkWidget          *entry;
} ent;

static void         set_text (GtkWidget * w, gpointer data);

static void         set_text (GtkWidget * w, gpointer data);

/*--------------------------------------------------------------------------*/

/*
 * a popup window, with a title, a message and an 
 * OK button. Rest of processing will be frozen
 * until OK is pressed. 
 */
void
message_box (char *title, char *fmt, ...)
{
    GtkWidget          *dialog;
    GtkWidget          *label;
    GtkWidget          *button;
    va_list             ap;
    char                msg[MESSAGESIZE];

    dialog = gtk_dialog_new ();

    /*
     * 'freeze' 
     */
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

    gtk_window_set_title (GTK_WINDOW (dialog), title);
    gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
    gtk_widget_set_usize (dialog, 0, 0);

    /*
     * Compose message 
     */
    memset ((void *) msg, 0, MESSAGESIZE);
    va_start (ap, fmt);
    vsnprintf (msg, MESSAGESIZE, fmt, ap);
    va_end (ap);

    /*
     * Show message 
     */
    label = gtk_label_new (msg);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label,
			FALSE, FALSE, 10);
    gtk_widget_show (label);

    /*
     * Ok Button 
     */
    button = gtk_button_new_with_label ("OK");
    set_button_default_size (button);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_main_quit),
			       GTK_OBJECT (dialog));
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_widget_destroy),
			       GTK_OBJECT (dialog));
    gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
		      FALSE, FALSE, 0);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (button);
    gtk_widget_show (button);
    gtk_widget_show (dialog);

    gtk_main ();
    return;
}

static void
set_text (GtkWidget * w, gpointer data)
{
    if (ent.text)
	free (ent.text);
    ent.text = strdup (gtk_entry_get_text (GTK_ENTRY (ent.entry)));
}

/*
 * show an entry text window with title and
 * message and returns the typed text or NULL if
 * cancel or no text was typed 
 */
char               *
entry_popup (char *title, char *fmt, ...)
{
    GtkWidget          *dialog;
    GtkWidget          *label;
    GtkWidget          *button;
    char                msg[MESSAGESIZE];
    va_list             ap;

    ent.text = NULL;
    dialog = gtk_dialog_new ();

    /*
     * 'freeze' 
     */
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

    gtk_window_set_title (GTK_WINDOW (dialog), title);
    gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
    gtk_widget_set_usize (dialog, 0, 0);

    /*
     * Compose message 
     */
    memset ((void *) msg, 0, MESSAGESIZE);
    va_start (ap, fmt);
    vsnprintf (msg, MESSAGESIZE, fmt, ap);
    va_end (ap);

    /*
     * Show message 
     */
    label = gtk_label_new (msg);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label,
			FALSE, FALSE, 10);
    gtk_widget_show (label);

    /*
     * The Entry Widget 
     */
    ent.entry = gtk_entry_new ();
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), ent.entry,
			FALSE, FALSE, 10);
    g_signal_connect (GTK_OBJECT (ent.entry), "activate",
		      G_CALLBACK (set_text), NULL);
    gtk_signal_connect_object (GTK_OBJECT (ent.entry), "activate",
			       G_CALLBACK (gtk_main_quit),
			       GTK_OBJECT (dialog));
    gtk_signal_connect_object (GTK_OBJECT (ent.entry), "activate",
			       G_CALLBACK (gtk_widget_destroy),
			       GTK_OBJECT (dialog));
    GTK_WIDGET_SET_FLAGS (ent.entry, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (ent.entry);
    gtk_widget_show (ent.entry);

    /*
     * OK button 
     */
    button = gtk_button_new_with_label ("OK");
    set_button_default_size (button);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (set_text), NULL);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_main_quit),
			       GTK_OBJECT (dialog));
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_widget_destroy),
			       GTK_OBJECT (dialog));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
			FALSE, FALSE, 0);
    gtk_widget_show (button);

    /*
     * Cancel Button 
     */
    button = gtk_button_new_with_label ("Cancel");
    set_button_default_size (button);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_main_quit),
			       GTK_OBJECT (dialog));
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_widget_destroy),
			       GTK_OBJECT (dialog));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
			FALSE, FALSE, 0);
    gtk_widget_show (button);
    gtk_widget_show (dialog);

    gtk_main ();
    return ent.text;
}

/*
 * confirm popup. Opens a window with title and
 * message, and two buttons, OK and cancel.
 * Returns OK or CANCEL 
 */
RETVALUE            value;

static void
setok (GtkWidget * w, gpointer data)
{
    value = OK;
}

static void
setcancel (GtkWidget * w, gpointer data)
{
    value = CANCEL;
}

RETVALUE
confirm (char *title, char *fmt, ...)
{
    GtkWidget          *dialog;
    GtkWidget          *label;
    GtkWidget          *button;
    char                msg[MESSAGESIZE];
    va_list             ap;

    dialog = gtk_dialog_new ();

    /*
     * 'freeze' 
     */
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

    gtk_window_set_title (GTK_WINDOW (dialog), title);
    gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
    gtk_widget_set_usize (dialog, 0, 0);

    /*
     * Compose message 
     */
    memset ((void *) msg, 0, MESSAGESIZE);
    va_start (ap, fmt);
    vsnprintf (msg, MESSAGESIZE, fmt, ap);
    va_end (ap);

    /*
     * Show message 
     */
    label = gtk_label_new (msg);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label,
			FALSE, FALSE, 10);
    gtk_widget_show (label);

    /*
     * OK button 
     */
    button = gtk_button_new_with_label ("OK");
    set_button_default_size (button);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (setok), NULL);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_main_quit),
			       GTK_OBJECT (dialog));
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_widget_destroy),
			       GTK_OBJECT (dialog));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
			FALSE, FALSE, 0);
    gtk_widget_show (button);

    /*
     * Cancel button 
     */
    button = gtk_button_new_with_label ("Cancel");
    set_button_default_size (button);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (setcancel), NULL);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_main_quit),
			       GTK_OBJECT (dialog));
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       G_CALLBACK (gtk_widget_destroy),
			       GTK_OBJECT (dialog));
    gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
		      FALSE, FALSE, 0);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (button);
    gtk_widget_show (button);
    gtk_widget_show (dialog);

    gtk_main ();

    return value;
}

void
status_bar_create (GtkWidget * w)
{
    status = gtk_statusbar_new ();
    gtk_box_pack_start (GTK_BOX (w), status, FALSE, FALSE, 2);
    status_bar_ctx_id =
	gtk_statusbar_get_context_id (GTK_STATUSBAR (status), "HPTALX");
    gtk_widget_show (status);

    status_bar_stack_height = 0;
}

void
status_push (char *fmt, ...)
{
    char                msg[MESSAGESIZE];
    va_list             ap;

    if (!status)		/* many happen during startup */
	return;

    memset ((void *) msg, 0, MESSAGESIZE);

    va_start (ap, fmt);
    vsnprintf (msg, MESSAGESIZE, fmt, ap);
    va_end (ap);

    gtk_statusbar_push (GTK_STATUSBAR (status), status_bar_ctx_id, msg);
    status_bar_stack_height++;

    while (gtk_events_pending ())
	gtk_main_iteration ();
}

void
status_clear ()
{
    while (status_bar_stack_height > 0) {
	status_bar_stack_height--;
	gtk_statusbar_pop (GTK_STATUSBAR (status), status_bar_ctx_id);
    }
}

void
status_pop ()
{
    if (status_bar_stack_height == 0)
	return;

    status_bar_stack_height--;
    gtk_statusbar_pop (GTK_STATUSBAR (status), status_bar_ctx_id);
}

void
set_button_default_size (GtkWidget * b)
{
    if (b == NULL)
	return;

    gtk_widget_set_usize (b, 100, 30);
}
