
/*
 * HPTalx
 * A PC<->HP48/49 communication program for Linux
 *
 * (C) 1999, 2000 by Bruno Barberi Gnecco
 * (C) 2005 by Rafael Ostertag
 *
 * $Id: hpcomm.h,v 1.3 2007/09/29 21:21:01 guengel Exp $
 *
 *
 *
 * Copyright note:
 *
 * This program is free software; you can
 * redistribute it and/or modify it under the
 * terms of the GNU General Public License as
 * published by the Free Software Foundation;
 * either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that
 * it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU
 * General Public License along with this
 * program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 */

#ifndef HPCOMM_H
#define HPCOMM_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "kermit.h"

#include <unistd.h>
#include <glib.h>

/*
 * Home directory of HP Calcs
 */
#define HP_HOME "{HOME}"

/*
 * The variable name used for storing the HP Calculator flags on the HP
 * Calculator 
 */
#define HP_FLAG_VARIABLE_NAME "$$$$flags"

/*
 * The variable name used to store the program for restoring the flags saved
 * in HP_FLAG_VARIABLE_NAME 
 */
#define HP_FLAG_RESTORE_NAME "fLaG"

/*
 * The type number of HP Objects
 *
 * See http://www.hpcalc.org/hp49/docs/misc/49g_aug.zip
 */
#define	HP_REAL 0
#define	HP_COMPLEX 1
#define	HP_STRING 2
#define	HP_REAL_ARRAY 3
#define	HP_COMPLEX_ARRAY 4
#define	HP_LIST 5
#define	HP_GLOBAL_NAME 6
#define	HP_LOCAL_NAME 7
#define	HP_PROGRAM 8
#define	HP_ALG_OBJECT 9
#define	HP_BINT 10
#define	HP_GRAPHICS_OBJECT 11
#define	HP_TAGGED_OBJECT 12
#define	HP_UNIT_OBJECT 13
#define	HP_XLIB_NAME 14
#define	HP_DIRECTORY 15
#define	HP_LIBRARY 16
#define	HP_BACKUP_OBJECT 17
#define	HP_REAL_INTEGER 28
#define	HP_FONT 30
#define	HP_BUILTIN_FUNCTION 18
#define	HP_BUILTIN_COMMAND 19
#define	HP_SYSTEM_BINARY 20
#define HP_EXTENDED_REAL 21
#define HP_EXTENDED_COMPLEX 22
#define HP_LINKED_ARRAY 23
#define HP_CHARACTER 24
#define HP_CODEOBJECT 25
#define HP_LIBRARY_DATA 26
#define HP_MINI_FONT 27
#define HP_INTEGER_NUMBER 28
#define HP_SYMBOLIC_VECTOR 29
#define HP_FONT 30
#define HP_EXTEND_OBJECT 21
#define HP_TYPE_UNKNOWN 999

/*
 * The HP Type names 
 */
#define HP_STR_REAL "real"		/* String indicating a Real number */

#define HP_STR_COMPLEX "complex"	/* String indicating a Complex number */

#define HP_STR_STRING "string"	/* String indicating a String */

#define HP_STR_REAL_ARRAY "real array"	/* String indicating an Array of Real 
										 * numbers */

#define HP_STR_COMPLEX_ARRAY "complex array"	/* String indicating an Array 
												 * of Complex numbers */

#define HP_STR_LIST "list"		/* String indicating a List */

#define HP_STR_GLOBAL_NAME "global"	/* String indicating a Global Name */

#define HP_STR_LOCAL_NAME "local name"	/* String indicating a Local Name */

#define HP_STR_PROGRAM "program"	/* String indicating a Program */

#define HP_STR_ALGEBRAIC_OBJECT "algebraic"	/* String indicating an Algebraic 
											 * Object */

#define HP_STR_BINARY_INTEGER "integer"	/* String indicating a Binary Integer 
										 */

#define HP_STR_GRAPHICS_OBJECT "graphics object"	/* String indicating a
													 * Graphics Object */

#define HP_STR_TAGGED_OBJECT "tagged object"	/* String indicating a Tagged 
												 * Object */

#define HP_STR_UNIT "unit"		/* String indicating an Unit */

#define HP_STR_XLIB "xlib"		/* String indicating a XLib */

#define HP_STR_DIRECTORY "directory"	/* String indicating a Directory */

#define HP_STR_LIBRARY "library"	/* String indicating a Library */

/*
 * Characters used throughout hpcomm.c 
 */
#define HP_CHAR_NEWLINE '\n'
#define HP_CHAR_SPACE ' '
#define HP_CHAR_DIR_START '{'
#define HP_CHAR_DIR_END '}'
#define HP_CHAR_NULL '\0'
#define HP_CHAR_SLASH '/'

/*
 * Strings used throughout hpcomm.c 
 */
#define HP_STR_NEWLINE "\n"
#define HP_STR_BACKSLASH "\\"
#define HP_STR_SLASH "/"
#define HP_STR_SPACE " "
#define HP_STR_DIR_START "{"
#define HP_STR_DIR_END "}"

/*
 * HP Calculator RPL Commands 
 */
#define HP_CMD_3TRANSIO "3 TRANSIO"
#define HP_CMD_CLEAR "CLEAR"
#define HP_CMD_VERSION "VERSION"
#define HP_CMD_SINGLE_EVAL "%s EVAL"
#define HP_CMD_MKDIR "'%s' CRDIR"
#define HP_CMD_PURGE_VAR "{ %s } PURGE"
#define HP_CMD_PURGE_DIR "{ %s } PGDIR"
#define HP_CMD_RECALL_FLAGS "RCLF"

/*
 * HP Calculator Model strings 
 */
#define HP_CALC_MODEL_48 "HP48"
#define HP_CALC_MODEL_49 "HP49"
#define HP_CALC_MODEL_49GPLUS "HP49-C"
#define HP_CALC_MODEL_50G "HP50-C"

/*
 * HP File Type strings 
 */
#define HP_FT_STR_HP48 "HP48"
#define HP_FT_STR_HP49 "HP49"
#define HP_FT_STR_HP49GPLUS "HP49-C"
#define HP_FT_STR_HP50G "HP50-C"	/* Just here for completeness. It is not
									 * really used, since the HP50g uses the
									 * same format as the HP49GPLUS. */
#define HP_FT_STR_ANY "%%HP"

/*
 * Maximum size of a directory name found on the HP Calculator 
 */
#define HP_DIRNAMESIZE 256

/*
 * Maximum size of a literal HP Type name 
 */
#define HP_TYPENAMESIZE 64

/*
 * Number of bytes read from the beginning of a file to determine what type
 * of HP File it is.
 */
#define HP_FILEHEADERSIZE 8

/*
 * The maximal stack number size. Explanation: When executing commands on the 
 * HP Calculator, it always returns the entire stack. Each line returned is
 * prefixed by its position on the HP Stack. So the first line is prefixed by 
 * "1:", the second line is prefixed by "2:" and so on. Using the size below, 
 * the max stack number can be "999:" plus the terminating '\0' which I guess 
 * is a huge stack, and very unlike to be encountered. 
 */
#define HP_STACKNUMBERSIZE 5

/*
 * This is the maximal stack number that can be stored with
 * HP_STACKNUMBERSIZE. Its calculated as follow: HP_STACKNUMBERSIZE - 2, 1
 * for ":" and 1 for '\0'. The result are the maximal digits. 
 */
#define HP_MAXSTACKNUMBER (pow(10,(HP_STACKNUMBERSIZE-2))-1)

/*
 * This is used to check the return value of the Kermit functions. If the
 * return value is not K_OK, it returns HP_KERMIT 
 */
#define HP_CHECK_KSTAT(a) if (a!=K_OK) return HP_KERMIT;

/*
 * This is used to check the return value of the hp_* functions. If the
 * return value is not HP_OK, it returns the argument to the macro 'a' 
 */
#define HP_CHECK_HPSTAT(a) if (a!=HP_OK) return a;

/*
 * Status codes that are returned by hp_* functions
 */
enum _hp_status
{
	HP_OK = 0,					/* Everything went OK */
	HP_INVARG,					/* Invalid arguments passed to the function */
	HP_KERMIT,					/* Kermit encountered an error */
	HP_XMODEM,					/* XModem encountered an error */
	HP_DIR,
	HP_MALLOC,					/* Malloc() encountered an error */
	HP_PARSE,					/* Error while parsing buffer */
	HP_BUFFER,					/* Buffer too small */
	HP_FOPEN,					/* File open failed */
	HP_FREAD					/* File read failed */
};
typedef enum _hp_status hp_status;

/*
 * Calculator type 
 */
enum _hp_calc
{
	HP48 = 0,					/* HP48 Calculator */

	HP49G,						/* HP49g Calculator */

	HP49G_PLUS,					/* HP49g+ Calculator */

	HP50G,						/* HP50g Calculator */

	HP_UNKNOWN					/* Unknown calculator */
};
typedef enum _hp_calc hp_calc;

/*
 * File types supported by HP Calculators
 */
enum _hp_file_type
{
	HP_FT_ANY = 0,				/* HP File that can be stored on each HP
								 * Calculator */

	HP_FT_HP48,					/* HP File that should only be stored on a
								 * HP48 Calculator */

	HP_FT_HP49,					/* HP File that should only be stored on a
								 * HP49 Calculator */

	/*
	 * HP_FT_HP50, 
	 *
	 * HP File that should only be stored on a HP50 Calculator, but HP is as
	 * lazzy as I am, so there is no new file type for the HP50g 
	 */

	HP_FT_UNKNOWN				/* Unknown file type, probably not a HP File */
};
typedef enum _hp_file_type hp_file_type;

/*
 * This is used for the hp_rename() function to indicate whether a directory
 * or a variable has to be renamed 
 */
enum _hp_rename_type
{
	HP_REN_DIR = 0,				/* Rename a directory */
	HP_REN_VAR					/* Rename a variable */
};
typedef enum _hp_rename_type hp_rename_type;

/*
 * This is used for the hp_purge() function to indicate whether a directory
 * or a variable has to be purged. 
 */
enum _hp_purge_type
{
	HP_PURGE_DIR = 0,			/* Purge a directory */
	HP_PURGE_VAR				/* Purge a variable */
};
typedef enum _hp_purge_type hp_purge_type;

/*
 * Data structure for the directory entries returned by hp_get_directory() 
 */
struct _hp_directory_entry
{
	char               *name;	/* Name of the entry */

	unsigned short      type;	/* Type of the entry see type number of HP
								 * Objects above */

	long                size;	/* Space needed by the entry on the
								 * Calculator */
};
typedef struct _hp_directory_entry hp_directory_entry;

extern hp_status    hp_version (hp_calc * v);

extern hp_status    hp_get_directory (const char *dn, GList ** list);

extern hp_status    hp_get_current_directory (char *dir, size_t s);

extern hp_status    hp_chdir (const char *hpdir);

extern hp_status    hp_mkdir (const char *dir);

extern hp_status    hp_get_file_version (const char *f, hp_file_type * v);

extern hp_status    hp_directory_unix_style (char *hpdir, char *ustyle,
											 size_t s);

extern hp_status    hp_directory_push (const char *hpdir, const char *d,
									   char *newdir, size_t s);

extern hp_status    hp_directory_pop (const char *hpdir, char *newdir,
									  size_t s);

extern hp_status    hp_rename (const char *oldname, const char *newname,
							   const hp_rename_type isdirectory);

extern hp_status    hp_purge (const char *name,
							  const hp_purge_type isdirectory);

extern hp_status    hp_send_file (const char *filename, const hp_file_type ft,
								  k_transfer_callback callback,
								  void *cb_data);

extern hp_status    hp_get_file (const char *name, k_transfer_type tt,
								 k_transfer_callback callback, void *cb_data);

extern hp_status    hp_copy_file (const char *name, const char *dest);

extern hp_status    hp_command (const char *command, char **output);

extern hp_status    hp_store_flags ();

extern hp_status    hp_purge_stored_flags ();

extern void         hp_cancel_transfer ();

#endif
