
/*
 *  HPTalx
 *  A PC<->HP48/49 communication program for Linux
 *
 *  (C) 1999, 2000 by Bruno Barberi Gnecco
 *  (C) 2005 by Bruno Barberi Gnecco, Rafael Ostertag
 *
 * rev 2.0: Added kermit/xmodem support
 *
 * $Id: hptalx.h,v 1.3 2007/09/29 21:21:01 guengel Exp $
 *
 *
 * Copyright note:
 *
 * This program is free software; you can
 * redistribute it and/or modify it under the
 * terms of the GNU General Public License as
 * published by the Free Software Foundation;
 * either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that
 * it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU
 * General Public License along with this
 * program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 */

#ifndef HPTALX_H
#define HPTALX_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtk.h>
#include <stdio.h>

#include "gui_utils.h"
#include "hpcomm.h"

/*
 * Buffer size
 */
#define BUFFERSIZE 256

enum connection_type
{
    SERIAL,
    TTYUSB
};
typedef enum connection_type CONNECTION_TYPE;

struct _pref
{
    int                 port;	/* COM1 = 0, 1,2, 3 */
    int                 speed;	/* Speed of port */
    int                 delete_confirm;
    int                 start_connected;
    int                 wait_connect;
    char               *editor;
};
typedef struct _pref Pref;

/*
 * This will be used globally. It is filled with the values selected by the
 * user from either a file or directory list. It does not take care of
 * multiple selections. This is in the responsibility of the function able to 
 * deal with multiple selections. 
 */
struct data
{
    char               *dir;	/* current selection value in dirs */
    char               *current;	/* current working directory */
    char               *filename;	/* current selection value in files */
    char               *file_type_str;	/* current type of the selection in * 
					 * files */
};

enum connection
{
    ONLINE, OFFLINE
};
typedef enum connection CONNECTION;

enum vpaned
{
    PC, HP, BOTH
};
typedef enum vpaned VPANED;

enum hpaned
{
    FILES, DIRS
};
typedef enum hpaned HPANED;

/*
 * Com speeds that are supported
 * Definition of speeds found in setup.c
 */
extern const char  *speed_list[];
extern hp_calc      hp_calculator;
extern struct data  data_pc;
extern struct data  data_hp;
extern VPANED       whereami_v;
extern HPANED       whereami_h;
extern CONNECTION   connected;
extern Pref         preferences;
extern k_transfer_type ttype;

#endif
